import json
import aiohttp
from nsqio.utils import _convert_to_str, get_logger


logger = get_logger()


class NsqHTTPConnection:
    """XXX"""

    def __init__(self, host="127.0.0.1", port=4150, *, loop):
        self._loop = loop
        self._endpoint = (host, port)
        self._base_url = "http://{0}:{1}/".format(*self._endpoint)

        self._session = aiohttp.ClientSession(
            connector=aiohttp.TCPConnector(), loop=self._loop
        )

    @property
    def endpoint(self):
        return "http://{0}:{1}".format(*self._endpoint)

    async def close(self):
        return await self._session.close()

    async def perform_request(self, method, url, params, body):
        _body = _convert_to_str(body) if body else body
        url = self._base_url + url

        # debug info for user to check if exception happens

        # let user decide if what to do with the aiohttp exceptions
        resp = await self._session.request(method, url, params=params, data=_body)

        resp_body = await resp.text()
        logger.debug(
            f"resp= > {resp_body}, \
                     {method}, {url}, {params}, {_body}, {type(_body)}"
        )
        try:
            response = json.loads(resp_body)
        except ValueError:
            return resp_body
        return response

    def __repr__(self):
        cls_name = self.__class__.__name__
        return "<{}: {}>".format(cls_name, self._endpoint)



from nsqio.http.base import NsqHTTPConnection


class NsqLookupd(NsqHTTPConnection):
    """
    :see: http://nsq.io/components/nsqlookupd.html
    """

    async def ping(self):
        """Monitoring endpoint.
        :returns: should return `"OK"`, otherwise raises an exception.
        """
        return await self.perform_request("GET", "ping", None, None)

    async def info(self):
        """Returns version information."""
        response = await self.perform_request("GET", "info", None, None)
        return response

    async def lookup(self, topic):
        """XXX

        :param topic:
        :return:
        """
        response = await self.perform_request("GET", "lookup", {"topic": topic}, None)
        return response

    async def topics(self):
        """XXX

        :return:
        """
        resp = await self.perform_request("GET", "topics", None, None)
        return resp

    async def channels(self, topic):
        """XXX

        :param topic:
        :return:
        """
        resp = await self.perform_request("GET", "channels", {"topic": topic}, None)
        return resp

    async def nodes(self):
        """XXX

        :return:
        """
        resp = await self.perform_request("GET", "nodes", None, None)
        return resp

    async def create_topic(self, topic):
        """XXX

        :param topic:
        :return:
        """
        resp = await self.perform_request(
            "POST", "/topic/create", {"topic": topic}, None
        )
        return resp

    async def delete_topic(self, topic):
        """XXX

        :param topic:
        :return:
        """
        resp = await self.perform_request(
            "POST", "/topic/delete", {"topic": topic}, None
        )
        return resp

    async def create_channel(self, topic, channel):
        """XXX

        :param topic:
        :param channel:
        :return:
        """
        resp = await self.perform_request(
            "POST", "/channel/create", {"topic": topic, "channel": channel}, None
        )
        return resp

    async def delete_channel(self, topic, channel):
        """XXX

        :param topic:
        :param channel:
        :return:
        """
        resp = await self.perform_request(
            "POST", "/channel/delete", {"topic": topic, "channel": channel}, None
        )
        return resp

    async def tombstone_topic_producer(self, topic, node):
        """XXX

        :param topic:
        :param node:
        :return:
        """
        resp = await self.perform_request(
            "POST", "delete_channel", {"topic": topic, "node": node}, None
        )
        return resp
